/**
 * lazy-hydration v1.0.2
 * (c) 2019 Rahul Kadyan <hi@znck.me> (https://znck.me)
 * @license MIT
 * @description Built with rollup-plugin-vue v4.7.2 and vue-template-compiler v2.6.10
 */
'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var PropTypes = _interopDefault(require('@znck/prop-types'));

/* eslint-disable no-console */
var isBrowser = typeof window !== 'undefined';
var io = typeof IntersectionObserver !== 'undefined' ? new IntersectionObserver(function (entries) {
  entries.forEach(function (entry) {
    if (entry.isIntersecting || entry.intersectionRatio > 0) {
      entry.target.dispatchEvent(new CustomEvent('hydration:visible'));
    }
  });
}) : null;
var asyncFactoryResolved = {
  resolved: true
};
var asyncFactory = {};
var clientSideDivData = {
  attrs: {
    'data-force-hydrate': true
  }
};
var Hydrate = {
  props: process.env.NODE_ENV !== 'production' ? {
    on: PropTypes.oneOfType(String, PropTypes.arrayOf(String)),
    onClick: PropTypes.bool,
    onHover: PropTypes.bool,
    onInteraction: PropTypes.bool,
    whenVisible: PropTypes.bool,
    whenIdle: PropTypes.bool,
    withDelay: PropTypes.number,
    ssrOnly: PropTypes.bool,
    force: PropTypes.bool
  } : ["on", "on", "onClick", "onHover", "onInteraction", "whenVisible", "whenIdle", "withDelay", "ssrOnly"],
  data: function data() {
    return {
      hydrated: !isBrowser
    };
  },
  created: function created() {
    var _this = this;

    if (process.env.NODE_ENV !== 'production') {
      PropTypes.validate(function () {
        if (!_this.on && !_this.onClick && !_this.onHover && !_this.onInteraction && !_this.whenVisible && !_this.whenIdle && !_this.ssrOnly && _this.force === undefined) {
          console.error("Select at least one trigger to enable hydration. If you don't want to hydrate at all use 'ssr-only'.");
        }

        if (_this.withDelay && _this.withDelay < 800) {
          console.warn("Delay duration ".concat(_this.withDelay, "ms is too low. A good choice would be around 2000ms. See https://github.com/znck/lazy-hydration."));
        }
      });
    }
  },
  mounted: function mounted() {
    var _this2 = this;

    if (this.$el.dataset.forceHydrate) {
      if (process.env.NODE_ENV !== 'production') {
        PropTypes.validate(function () {
          console.log('No SSR rendered content found. Force Hydrate.');
        });
      } // No SSR rendered content. Render now.


      this.hydrate();
      return;
    }

    if (this.ssrOnly) return;
    var withDelay;
    var on = (Array.isArray(this.on) ? this.on : [this.on]).slice().filter(function (name) {
      return typeof name !== 'string';
    });

    if (this.onClick) {
      on.push('click');
    }

    if (this.onHover || this.onInteraction) {
      on.push('mouseenter');

      if (this.onInteraction) {
        on.push('focus');
      }
    }

    if (this.whenIdle) {
      if (typeof requestIdleCallback !== 'undefined') {
        var id = requestIdleCallback(function () {
          requestAnimationFrame(function () {
            _this2.hydrate();
          });
        }, {
          timeout: 500
        });

        this.idle = function () {
          return cancelIdleCallback(id);
        };
      } else withDelay = 2000;
    }

    if (this.whenVisible) {
      var el = this.$el; // As root node does not have any box model, it cannot intersect.

      on.push('hydration:visible');
      if (io) io.observe(el);else {
        withDelay = 2000;

        if (process.env.NODE_ENV !== 'production') {
          PropTypes.validate(function () {
            return console.warn('IntersectionObserver polyfill is required.');
          });
        }
      }

      this.visible = function () {
        io && io.unobserve(el);
      };
    }

    if (on.length) {
      on.forEach(function (event) {
        return _this2.$el.addEventListener(event, _this2.hydrate, {
          capture: true,
          once: true
        });
      });

      this.off = function () {
        return on.forEach(function (event) {
          return _this2.$el.removeEventListener(event, _this2.hydrate);
        });
      };
    }

    if (this.withDelay || withDelay) {
      var _id = setTimeout(this.hydrate, this.withDelay || withDelay);

      this.delay = function () {
        return clearTimeout(_id);
      };
    }
  },
  beforeDestroy: function beforeDestroy() {
    this.cleanup();
  },
  methods: {
    cleanup: function cleanup() {
      var handlers = ['visible', 'idle', 'delay', 'off'];

      for (var _i = 0, _handlers = handlers; _i < _handlers.length; _i++) {
        var handler = _handlers[_i];

        if (handler in this) {
          this[handler]();
          delete this[handler];
        }
      }
    },
    hydrate: function hydrate() {
      this.hydrated = true;
      this.cleanup();
    }
  },
  render: function render(h) {
    var _this3 = this;

    var firstChild = function firstChild() {
      var children = _this3.$scopedSlots.default ? _this3.$scopedSlots.default({
        hydrated: _this3.hydrated
      }) : _this3.$slots.default;
      return Array.isArray(children) ? children[0] : children;
    };

    var vnode = this.hydrated ? firstChild() : h('div', clientSideDivData);
    vnode.asyncFactory = this.hydrated ? asyncFactoryResolved : asyncFactory;
    vnode.isComment = !this.hydrated;

    if (isBrowser) {
      window.vnode = vnode;
    }

    return vnode;
  },
  watch: {
    force: {
      handler: function handler(value) {
        if (value) this.hydrate();
      },
      immediate: true
    }
  }
};

module.exports = Hydrate;
